------------- I n p u t _ u n i t ------------------------------------------
-- This code makes it easy to verify the funcionality of the ALU
-- How to use:
--  1) If the leftmost Dp and Dig1:0 (the rigthmost) flashes
--     does it mean the you can define A(7:0) with the Sw(7:0)
--     Press <Btn1> in order to update => A = Dig1:0 = Dig3:2
--
--  2) Press <Btn0> in order to change input "Valg"
--
--  3) If the next leftmost Dp and Dig1:0 (the rigthmost) flashes
--     does it mean the you can define B(7:0) with the Sw(7:0)
--     Press <Btn1> in order to update => B = Dig1:0 = Dig3:2
--
--  4) Press <Btn0> again in order to change "Valg" for arithmatic
--
--  5) When Dig3 flashes for arithmatic mode.
--     Use <Btn3> and <Btn2> to select S = 0..7
--     Use <Btn2> to toggle the Cin (watch Dp0)
--  6) Press <Btn0> to start the process of watch/change A,B  
--  7) Press <Btn0> to watch the 16-bit F value
---------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Input_Unit is
    Port (  Clk_50MHz :	IN  STD_LOGIC;
				Sw : 			IN  STD_LOGIC_VECTOR (7 downto 0);
            Btn : 		IN  STD_LOGIC_VECTOR (3 downto 0);
				Ld: 		   OUT std_logic_vector(7 downto 0);
			  	Segm : 		OUT std_logic_vector(1 to 8);
				An : 			OUT std_logic_vector(3 downto 0));
end Input_Unit;

architecture Behavioral of Input_Unit is
   ----------------------------------------------------------------------
	COMPONENT ALU_Design
	PORT(
		Cin :       IN  std_logic;
		S :   		IN  std_logic_vector(2 downto 0);
		A,B:			IN  std_logic_vector(7 downto 0);
		F,hiF :	   OUT std_logic_vector(7 downto 0);
		Cout: 		OUT std_logic);
	END COMPONENT;	
	Signal A,B,F,hiF : STD_LOGIC_VECTOR (7 downto 0);
	Signal BxSw:      STD_LOGIC_VECTOR (7 downto 0);
   Signal S : 	    	STD_LOGIC_VECTOR (2 downto 0);
   Signal Cout,Cin :	STD_LOGIC;
   ----------------------------------------------------------------------
	COMPONENT Muxdisplay_ver5e
	PORT(
		Clk_50MHz : IN  std_logic;
		Blank : 		IN  std_logic_vector(3 downto 0);
		Hex : 		IN  std_logic_vector(15 downto 0);
		Dp : 			IN  std_logic_vector(3 downto 0);          
		Segm : 		OUT std_logic_vector(1 to 8);
		An : 			OUT std_logic_vector(3 downto 0);
		Clk_1kHz : 	OUT std_logic;
		Clk_1Hz : 	OUT std_logic);
	END COMPONENT;
	Signal Blank :		std_logic_vector(3 downto 0); 
	Signal Hex :  		std_logic_vector(15 downto 0);
	Signal Dp :   		std_logic_vector(3 downto 0);          
	Signal Clk_1kHz : std_logic;
	Signal Clk_1Hz :  std_logic;	
	----------------------------------------------------------------------
	Signal Valg:      STD_LOGIC_VECTOR ( 1 downto 0) := "00"; 	

begin
   -- OPGAVE lav en 8-bit version af 74x382 - Wak. side 485 ------------- 
   ALU:  ALU_design PORT MAP(
		Cin 	=> Cin,
		S		=> S,
		A		=> A,
		B		=> BxSw,
		F		=> F,
		hiF	=> hiF,
		Cout	=> Cout);

   BxSw <= Sw when Valg="11" else B; -- Mux for B or Sw selection
	
   -- Opdateret version af Mux-display -----------------------------------
	Disp: Muxdisplay_ver5e PORT MAP(
		Clk_50MHz =>  Clk_50MHz,
		Blank     =>  Blank   ,
		Hex 		 =>  Hex 	  ,	
		Dp 		 =>  Dp 		  ,
		Segm 		 =>  Segm 	  ,	
		An 		 =>  An 		  ,
		Clk_1kHz  =>  Clk_1kHz , 
		Clk_1Hz   =>  Clk_1Hz);
	
   Bestemmelse_af_hvad_der_skal_vises_paa_display:
	------------------------------------------------------------------------
	process (Valg,Sw,A,B,Cout,F,hiF,S,Blank,Clk_1HZ)
	begin
	   Ld  	<= (others=>'0');
		Hex 	<= (others=>'0');
		Dp  	<= "0000";
		Blank <= "0000";
	   case Valg is
		   When "00" =>
					Ld 	<= Sw;
					Hex   <= Sw&A;
					Dp    <= "0111";
					Blank <= "00"&Clk_1Hz&Clk_1Hz;		
			When "01" =>
					Ld 	<= Sw;
					Hex 	<= Sw&B;
					Dp  	<= "1011";
					Blank <= "00"&Clk_1Hz&Clk_1Hz;						
			When "10" =>
					Ld		<= F;
					Hex   <= '0'&S&"000"&Cout&F;
					Dp  	<= "111"& not Cin;	
					Blank <= Clk_1Hz&"000";	
			When "11" =>
					Ld		<= F;
					Hex   <= hiF & F;
					Dp  	<= not Cout & Clk_1Hz & not Clk_1Hz & not Cin;	
					Blank <= "0000";	
			when others =>
					Null;
		end case;
	End process;				
	
	Prelfang_og_detektion_af_Botton_tryk:
	------------------------------------------------------------------------
	process( Clk_50MHz)
	   Variable Shreg0: STD_LOGIC_VECTOR ( 15 downto 0) := (others=>'0');
	   Variable Shreg1: STD_LOGIC_VECTOR ( 15 downto 0) := (others=>'0');
	   Variable Shreg2: STD_LOGIC_VECTOR ( 15 downto 0) := (others=>'0');
	   Variable Shreg3: STD_LOGIC_VECTOR ( 15 downto 0) := (others=>'0');
	begin
	   if rising_edge( Clk_50MHz) then
		   if Clk_1kHz='1' then
				Shreg0 := Shreg0(14 downto 0)&Btn(0);
				Shreg1 := Shreg1(14 downto 0)&Btn(1);
				Shreg2 := Shreg2(14 downto 0)&Btn(2);
				Shreg3 := Shreg3(14 downto 0)&Btn(3);
				----------------Tryk p Btn0 --------------				
				if Shreg0="0111111111111111" then
					if Valg = "11" then
						Valg <= "00";
					else
						Valg <= Valg+1; -- Skif imellem "Valg"
					end if;
				end if;
				----------------Tryk p Btn1 --------------
				if Shreg1="0111111111111111" then
					case Valg is
						when "00"=>
							A <= Sw;
						when "01"=>
							B <= Sw;
						when "10"|"11"=>
							Cin <= not Cin;
						when others =>
							null;					
					end case;
				end if;
				----------------Tryk p Btn2 ---------------
				if Shreg3="0111111111111111" then
					S <= S+1;  -- Vlg den nste funktion
				end if;
				----------------Tryk p Btn2 ---------------
				if Shreg2="0111111111111111" then
					S <= S-1;  -- Vlg den forrige funktion
				end if;
			end if;
		end if;
	end process;

end Behavioral;

