-- Extended version of the Mux Display driver - blanking of digits
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Muxdisplay_ver5e is
	PORT(
		Clk_50MHz : IN  std_logic;
		Blank : 		IN  std_logic_vector(3 downto 0);
		Hex : 		IN  std_logic_vector(15 downto 0);
		Dp : 			IN  std_logic_vector(3 downto 0);          
		Segm : 		OUT std_logic_vector(1 to 8);
		An : 			OUT std_logic_vector(3 downto 0);
		Clk_1kHz : 	OUT std_logic;
		Clk_1Hz : 	OUT std_logic);
end Muxdisplay_ver5e;

architecture Behavioral of Muxdisplay_ver5e is
begin
	---------------------------------------------------------------------
	Mux_Display:
	process( Clk_50MHz)
	   variable Scale50000:	integer range 0 to 50000; 
		variable Scale1023:  std_logic_vector( 9 downto 0);
		variable X:    		integer range 0 to 3 := 0;
		variable HexDig: 		std_logic_vector( 3 downto 0);
		type ROM_array is array (0 to 15) of std_logic_vector (1 to 7);
													--	 abcdefg
		constant Hex27Segm: ROM_array := (	"0000001", -- 0
														"1001111", -- 1
														"0010010", -- 2
														"0000110", -- 3
														"1001100", -- 4
														"0100100", -- 5
														"0100000", -- 6
														"0001111", -- 7
														"0000000", -- 8
														"0000100", -- 9
														"0001000", -- A
														"1100000", -- b
														"0110001", -- C
														"1000010", -- d
														"0110000", -- E
														"0111000");-- F
	begin
	   if rising_edge( Clk_50MHz) then
		   if Scale50000 <50000 then
			   Scale50000 := Scale50000 + 1;
				Clk_1kHz   <= '0';
			else
			   Scale50000 := 1;
				Clk_1kHz   <= '1';
				Scale1023  :=Scale1023+1;
				Clk_1Hz    <= Scale1023(9); -- Approx. 1 Hz
					
				if X<3 then
					X := X+1;
				else
					X := 0;
				end if;
				HexDig   := Hex( X*4+3 downto X*4);
				if Blank(X)='1' then
					Segm 	<= "1111111" & Dp(X);
				else
					Segm 	<= Hex27Segm( Conv_integer(HexDig)) & Dp(X);
				end if;
				An       <= "1111";
				An(x)    <= '0';
				
			end if;				   
      end if;
	end process;
end Behavioral;

